/*
 * GStreamer
 * Copyright (C) 2005 Thomas Vander Stichele <thomas@apestaart.org>
 * Copyright (C) 2005 Ronald S. Bultje <rbultje@ronald.bitfreak.net>
 * Copyright (C) 2010 Denis Schulze <<user@hostname.org>>
 * Copyright (C) 2011 Florian Lier <<flier@techfak.uni-bielefeld.de>>
 * Copyright (C) 2011 Jan Moringen <jmoringe@techfak.uni-bielefeld.de>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * Alternatively, the contents of this file may be used under the
 * GNU Lesser General Public License Version 2.1 (the "LGPL"), in
 * which case the following provisions apply instead of the ones
 * mentioned above:
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <iostream>

#include "gst_nao_audio_import.h"

GST_DEBUG_CATEGORY_STATIC (gst_nao_audio_import_debug);
#define GST_CAT_DEFAULT gst_nao_audio_import_debug
#define DEFAULT_PUB_NAME "gstnaoaudioimport"

/* Filter signals and args */
enum {
  LAST_SIGNAL
};

enum {
  PROP_0,
  PROP_PUB_NAME,
};

const gchar *scope;

// ==========================================================================
// Input and output pads for the plugin.
// ==========================================================================

static GstStaticPadTemplate nao_audio_import_sink_factory = GST_STATIC_PAD_TEMPLATE (
   "sink",
    GST_PAD_SINK,
    GST_PAD_ALWAYS,
    GST_STATIC_CAPS ("ANY")
    );

static GstStaticPadTemplate nao_audio_import_src_factory = GST_STATIC_PAD_TEMPLATE (
    "src",
    GST_PAD_SRC,
    GST_PAD_ALWAYS,
    GST_STATIC_CAPS ("ANY")
    );

GST_BOILERPLATE (GstNaoAudioImport, gst_nao_audio_import, GstElement, GST_TYPE_ELEMENT);

static void gst_nao_audio_import_set_property (GObject * object, guint prop_id, const GValue * value, GParamSpec * pspec);
static void gst_nao_audio_import_get_property (GObject * object, guint prop_id, GValue * value, GParamSpec * pspec);

static gboolean gst_nao_audio_import_set_caps (GstPad * pad, GstCaps * caps);
static GstFlowReturn gst_nao_audio_import_chain (GstPad * pad, GstBuffer * buf);

// ==========================================================================
// Initialization methods for plugin.
// ==========================================================================

static void gst_nao_audio_import_base_init (gpointer gclass) {
  GstElementClass *element_class = GST_ELEMENT_CLASS (gclass);
  gst_element_class_set_details_simple(element_class,
    "rsb_nao_audio_import",
    "",
    "Takes input from a gstreamer plugin and imports the buffer via rsb.",
    "Florian Lier <flier@techfak.uni-bielefeld.de>, Denis Schulze <dschulze@techfak.uni-bielefeld.de>");
}

static void gst_nao_audio_import_class_init (GstNaoAudioImportClass * klass) {
  GObjectClass *gobject_class = (GObjectClass *) klass;
  gobject_class->set_property = gst_nao_audio_import_set_property;
  gobject_class->get_property = gst_nao_audio_import_get_property;

  GParamSpec *pubNameSpec = g_param_spec_string("pubName",
						"publisher Name",
						"The scope used by the informer.",
						DEFAULT_PUB_NAME,
						(GParamFlags)G_PARAM_READWRITE);
  g_object_class_install_property(gobject_class, PROP_PUB_NAME, pubNameSpec);
}

// ==========================================================================
// Constructor.
// ==========================================================================

static gboolean gst_nao_audio_import_sink_event(GstPad *pad, GstEvent *event) {
    gst_event_ref(event);

    int ret;
    switch (GST_EVENT_TYPE(event)) {
    case GST_EVENT_NEWSEGMENT: {
	std::cout << "[Audio] new segment time " << GST_EVENT_TIMESTAMP(event) << std::endl;
	GstNaoAudioImport *rsbImporter = GST_NAO_AUDIO_IMPORT(gst_pad_get_parent (pad));
	rsbImporter->importer->setTime(GST_EVENT_TIMESTAMP(event));
    }
    default:
	ret = gst_pad_event_default(pad, event);
    };

    gst_event_unref(event);
    return ret;
}

/* initialize the new element
 * instantiate pads and add them to element
 * set pad calback functions
 * initialize instance structure
 */
static void gst_nao_audio_import_init (GstNaoAudioImport *rsbImport, GstNaoAudioImportClass *gclass) {
    rsbImport->sinkpad = gst_pad_new_from_static_template(&nao_audio_import_sink_factory, "sink");
    rsbImport->srcpad  = gst_pad_new_from_static_template(&nao_audio_import_src_factory,  "src");

    gst_pad_set_setcaps_function (rsbImport->sinkpad, GST_DEBUG_FUNCPTR(gst_nao_audio_import_set_caps));
    gst_pad_set_getcaps_function (rsbImport->sinkpad, GST_DEBUG_FUNCPTR(gst_pad_proxy_getcaps));
    gst_pad_set_chain_function   (rsbImport->sinkpad, GST_DEBUG_FUNCPTR(gst_nao_audio_import_chain));

    gst_element_add_pad(GST_ELEMENT(rsbImport), rsbImport->sinkpad);
    gst_element_add_pad(GST_ELEMENT(rsbImport), rsbImport->srcpad);

    gst_pad_set_event_function(rsbImport->sinkpad, &gst_nao_audio_import_sink_event);

    rsbImport->importer = new NaoAudioImporter();
}

static void gst_nao_audio_import_set_property(GObject * object, guint prop_id, const GValue * value, GParamSpec * pspec)
{
  GstNaoAudioImport *filter = GST_NAO_AUDIO_IMPORT (object);
  const gchar *name;
  switch (prop_id) {
         case PROP_PUB_NAME:
	      name = g_value_get_string(value);
	      scope = name;
	      filter->importer->setScopeName(scope);
	      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
      break;
  }
}

static void gst_nao_audio_import_get_property(GObject * object, guint prop_id, GValue * value, GParamSpec * pspec)
{
  GstNaoAudioImport *filter = GST_NAO_AUDIO_IMPORT(object);
  switch (prop_id) {
	case PROP_PUB_NAME:
	std::cout << "pubNAME " << std::endl;
	break;
  default:
	G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
	break;
  }
}

/* GstElement vmethod implementations */

/* this function handles the link with other elements */
static gboolean gst_nao_audio_import_set_caps(GstPad * pad, GstCaps * caps) {
  GstNaoAudioImport *filter;
  GstPad *otherpad;
  filter = GST_NAO_AUDIO_IMPORT (gst_pad_get_parent (pad));
  otherpad = (pad == filter->srcpad) ? filter->sinkpad : filter->srcpad;
  gst_object_unref (filter);
  return gst_pad_set_caps (otherpad, caps);
}

/* chain function
 * this function does the actual processing
 */
static GstFlowReturn gst_nao_audio_import_chain (GstPad * pad, GstBuffer * buf)
{
    GstNaoAudioImport *rsbImporter = GST_NAO_AUDIO_IMPORT(GST_OBJECT_PARENT (pad));

    rsbImporter->importer->processBuffer(buf);

    return gst_pad_push (rsbImporter->srcpad, buf);
}


/* entry point to initialize the plug-in
 * initialize the plug-in itself
 * register the element factories and other features
 */
static gboolean rsbimport_init (GstPlugin * rsbimport)
{

  /* debug category for fltering log messages
   *
   * exchange the string 'Template rsbimport' with your description
   */
  GST_DEBUG_CATEGORY_INIT (gst_nao_audio_import_debug, "rsb_nao_audio_import", 0, "Template Nao audio import");
  return gst_element_register (rsbimport, "rsb_nao_audio_import", GST_RANK_NONE, GST_TYPE_NAO_AUDIO_IMPORT);
}


/* PACKAGE: this is usually set by autotools depending on some _INIT macro
 * in configure.ac and then written into and defined in config.h, but we can
 * just set it ourselves here in case someone doesn't use autotools to
 * compile this code. GST_PLUGIN_DEFINE needs PACKAGE to be defined.
 */
#ifndef PACKAGE
#define PACKAGE "rsb_nao_audio_import"
#endif


/* gstreamer looks for this structure to register rsbimports
 *
 * exchange the string 'Template rsbimport' with your rsbimport description
 */
GST_PLUGIN_DEFINE (
    GST_VERSION_MAJOR,
    GST_VERSION_MINOR,
    "rsb_nao_audio_import",
    "Consumers passing byte[]buffer in a GSTREAMER pipeline (see: Identity Element) via RSB Author: flier@techfak.uni-bielefeld.de ",
    rsbimport_init,
    VERSION,
    "LGPL",
    "GStreamer",
    "http://gstreamer.net/"
)
